unit MPCodeASM;

interface
  uses Math;
Type
  TSingleMatrix    = Array of Array of Single;
  TMOperator       = Procedure(var M1, M2 : TSingleMatrix);
  TMCreator        = Function (Value      : Single;
                               XDim, YDim : Integer) : TSingleMatrix;
  TMFunction       = Procedure(var M1     : TSingleMatrix);

  Function SMatrix(Value : Single; XDim, YDim : Integer) : TSingleMatrix;
  Function XMatrix(Value : Single; XDim, YDim : Integer) : TSingleMatrix;
  Function YMatrix(Value : Single; XDim, YDim : Integer) : TSingleMatrix;

  Procedure MNorm (var M1 : TSingleMatrix);
  Procedure MRound(var M1 : TSingleMatrix);
  Procedure MTrunc(var M1 : TSingleMatrix);
  Procedure MAbs  (var M1 : TSingleMatrix);
  Procedure MSqr  (var M1 : TSingleMatrix);
  Procedure MSqrt (var M1 : TSingleMatrix);
  Procedure MExp  (var M1 : TSingleMatrix);
  Procedure MLn   (var M1 : TSingleMatrix);
  Procedure MSin  (var M1 : TSingleMatrix);
  Procedure MCos  (var M1 : TSingleMatrix);
  Procedure MTan  (var M1 : TSingleMatrix);
  Procedure MASin (var M1 : TSingleMatrix);
  Procedure MACos (var M1 : TSingleMatrix);
  Procedure MATan (var M1 : TSingleMatrix);

  Procedure MAdd(var M1, M2 : TSingleMatrix);
  Procedure MSub(var M1, M2 : TSingleMatrix);
  Procedure MMul(var M1, M2 : TSingleMatrix);
  Procedure MDiv(var M1, M2 : TSingleMatrix);
  Procedure MMod(var M1, M2 : TSingleMatrix);
  Procedure MPow(var M1, M2 : TSingleMatrix);
  Procedure MLog(var M1, M2 : TSingleMatrix);
  Procedure MMin(var M1, M2 : TSingleMatrix);
  Procedure MMax(var M1, M2 : TSingleMatrix);

implementation

Function SMatrix(Value : Single; XDim, YDim : Integer) : TSingleMatrix;
var
  x, y : Integer;
Begin
  Setlength(Result, YDim, XDim);
  For y := high(Result) downto 0 do
    For x := high(Result[0]) downto 0 do
      Result[y, x] := Value;
end;

Function XMatrix(Value : Single; XDim, YDim : Integer) : TSingleMatrix;
var
  x, y : Integer;
Begin
  Setlength(Result, YDim, XDim);
  For y := high(Result) downto 0 do
    For x := high(Result[0]) downto 0 do
      Result[y, x] := x;
end;

Function YMatrix(Value : Single; XDim, YDim : Integer) : TSingleMatrix;
var
  x, y : Integer;
Begin
  Setlength(Result, YDim, XDim);
  For y := high(Result) downto 0 do
    For x := high(Result[0]) downto 0 do
      Result[y, x] := y;
end;

Procedure MNorm(var M1 : TSingleMatrix);
var
  x, y     : Integer;
  min, max : Single;
Begin
  max := -maxSingle;
  min := maxSingle;
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
    Begin
      If M1[y, x] < min then
        min := M1[y, x];
      If M1[y, x] > max then
        max := M1[y, x];
    end;
  if max - min = 0 then
    max := max + 0.0000001;
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      M1[y, x] := ((M1[y, x] - min) / (max - min));
end;

Procedure MRound(var M1 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      M1[y, x] := Round(M1[y, x]);
end;

Procedure MTrunc(var M1 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      M1[y, x] := Trunc(M1[y, x]);
end;

Procedure MAbs(var M1 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      M1[y, x] := Abs(M1[y, x]);
end;


Procedure MSqr(var M1 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      M1[y, x] := Sqr(M1[y, x]);
end;

Procedure MSqrt(var M1 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      M1[y, x] := Sqrt(M1[y, x]);
end;

Procedure MExp(var M1 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      M1[y, x] := Exp(M1[y, x]);
end;

Procedure MLn(var M1 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      M1[y, x] := Ln(M1[y, x]);
end;

Procedure MSin(var M1 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      M1[y, x] := Sin(M1[y, x]);
end;

Procedure MCos(var M1 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      M1[y, x] := Cos(M1[y, x]);
end;

Procedure MTan(var M1 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      M1[y, x] := Tan(M1[y, x]);
end;

Procedure MASin(var M1 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      M1[y, x] := ArcSin(M1[y, x]);
end;

Procedure MACos(var M1 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      M1[y, x] := ArcCos(M1[y, x]);
end;

Procedure MATan(var M1 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      M1[y, x] := ArcTan(M1[y, x]);
end;











Procedure MAdd(var M1, M2 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      M1[y, x] := M1[y, x] + M2[y, x];
end;

Procedure MSub(var M1, M2 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      M1[y, x] := M1[y, x] - M2[y, x];
end;

Procedure MMul(var M1, M2 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      M1[y, x] := M1[y, x] * M2[y, x];
end;

Procedure MDiv(var M1, M2 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      M1[y, x] := M1[y, x] / M2[y, x];
end;


Procedure MMod(var M1, M2 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      M1[y, x] := M1[y, x] / M2[y, x] - trunc(M1[y, x] / M2[y, x]);
end;

Procedure MPow(var M1, M2 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      M1[y, x] := power(M1[y, x], M2[y, x]);
end;

Procedure MLog(var M1, M2 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      M1[y, x] := LogN(M1[y, x], M2[y, x]);
end;

Procedure MMin(var M1, M2 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      If M1[y, x]  > M2[y, x] then
         M1[y, x] := M2[y, x];
end;


Procedure MMax(var M1, M2 : TSingleMatrix);
var
  x, y : Integer;
Begin
  For y := high(M1) downto 0 do
    For x := high(M1[0]) downto 0 do
      If M1[y, x]  < M2[y, x] then
         M1[y, x] := M2[y, x];
end;


end.
