unit perlins;
interface
uses graphics, Interpreter, MPCodeASM, math;

type
//  TSingleMatrix  = Array of Array of Single;
  TSingleMatrix5 = Array[0..4] of Array[0..4] of Single;
  TSingleArray5  = Array[0..4] of Single;

  TResizeMethod  = (RMLinear, RMQuadratic, RMCosinus, RMCubic);
  TAddMode       = (AMSigned, AMNormalized, AMUnsigned , AMAbs);
  PByteArray     = ^TByteArray;
  TByteArray     = array[0..32767] of Byte;
  TPerlinFunc    = Procedure(Arguments  : Array of Single;
                             var Matrix : TSingleMatrix;
                             Formula    : String);
  TPerlFunc      = Record
    Func         : TPerlinFunc;
    Arguments    : Array[0..2] of Single;
    Formular     : String;
  end;
Function Rand1Db(x    : Integer) : Byte;
Function Rand2Db(x, y : Integer) : Byte;
Function Rand1Df(x    : Integer) : Single;
Function Rand2Df(x, y : Integer) : Single;

{Argument1 = cover, Argument2 = Sharpness}
Procedure Fexponentize(Arguments  : Array of Single;
                       var Matrix : TSingleMatrix;
                       Formula    : String);
{Argument1 = Matrix, Argument2 = X, Argument3 = y}
Procedure FMarble(Arguments  : Array of Single;
                  var Matrix : TSingleMatrix;
                  Formula    : String);
{Argument1 = Factor}
Procedure FWood(Arguments  : Array of Single;
                var Matrix : TSingleMatrix;
                Formula    : String);
{Argument1 =  Factor [0, 0], Argument2 = Factor [-1/1, 0]/[0, -1/1],
 Argument3 =  Factor [-1/1, -1/1]}
Procedure FSmooth(Arguments : Array of Single;
                 var Matrix : TSingleMatrix;
                 Formula    : String);

{Argument1 =  New X, Argument2 = New Y,
 Argument3 =  ResizeMode (0 = Cubic, 1 = Cosinus, 2 = Quadratic, 3 = Linear)}
Procedure FResize(Arguments  : Array of Single;
                  var Matrix : TSingleMatrix;
                  Formula    : String);

Procedure FInvert(Arguments : Array of Single;
                 var Matrix : TSingleMatrix;
                 Formula    : String);


Procedure FNone(Arguments  : Array of Single;
                var Matrix : TSingleMatrix;
                Formula    : String);

Procedure FAdvanced(Arguments  : Array of Single;
                    var Matrix : TSingleMatrix;
                    Formula    : String);

Function ClampToByte(Value : Integer) : Byte;
Function ClampToNorm(Value : Single)  : Single;
Function BiLinResize (Matrix  : TSingleMatrix;
                                  newWidth, newHeight  : Cardinal) : TSingleMatrix;
Function BiQuadResize(Matrix  : TSingleMatrix;
                                  newWidth, newHeight  : Cardinal) : TSingleMatrix;
Function BiCosResize (Matrix  : TSingleMatrix;
                                  newWidth, newHeight  : Cardinal) : TSingleMatrix;
Function BiCubResize (Matrix  : TSingleMatrix;
                                  newWidth, newHeight  : Cardinal) : TSingleMatrix;
Procedure smoothMatrix (var Matrix : TSingleMatrix; weighten : Single);
Procedure smoothMatrixM(var Matrix : TSingleMatrix; weighten : Array of Single);
Procedure NoiseMatrix  (var Matrix : TSingleMatrix; Seed     : Integer);
Function MatrixToBmp   (    Matrix : TSingleMatrix; Offset   : Byte; PF : TPixelformat;
                            Palette: Array of Cardinal   ) : TBitmap;
Function BmpToMatrix(Bmp : TBitmap; Offset : Byte; PF : TPixelformat;
                     Palette: Array of Cardinal ) : TSingleMatrix;

Function ValInArray(       Value : Integer; Arr : Array of Integer;
                    var Position : Integer)     : Boolean;
Function PerlinNoise(Persistance      : Single   ; StartOctave, Octaves : Byte;
                     Seed             : Integer  ;
                     PerlinFunc       : Array of TPerlFunc;
                     Layers           : Array of TSingleMatrix;
                     LayerOctaves     : Array of Integer;
                     Smooth_Weighten  : Array of Single;
                     Smooth           : Boolean       = False;


                     Translate        : Single        = 0;
                     Scale            : Single        = 1;
                     ResizeMethod     : TResizeMethod = RMCosinus;
                     AddMode          : TAddMode      = AMSigned)    : TSingleMatrix;

{Function PerlinNoiseBmp(Persistance     : Single  ; StartOctave, Octaves : Byte;
                        Seed            : Integer ; ResizeMethod         : TResizeMethod = RMCosinus;
                        Smooth          : Boolean = False;
                        Smooth_Weighten : Single  = 1)                   : TBitmap;
}
Function PerlinNoiseBmpE(Persistance      : Single   ; StartOctave, Octaves : Byte;
                     Seed             : Integer  ;
                    { FunctionArguments: Array of Single;}
                     Palette          : Array of Cardinal;
                     PerlinFunc       : Array of TPerlFunc;
                     Bmps             : Array of TBitmap;
                     BmpOctaves       : Array of Integer;
                     Smooth_Weighten  : Array of Single;
                     Smooth           : Boolean       = False;

                     Translate        : Single        = 0;
                     Scale            : Single        = 1;
                     ResizeMethod     : TResizeMethod = RMCosinus;
                     AddMode          : TAddMode      = AMSigned)         : TBitmap;

{Function PerlinNoise(Persistance     : Single   ; StartOctave, Octaves : Byte;
                     Seed            : Integer  ; ResizeMethod         : TResizeMethod = RMCosinus;
                     Smooth          : Boolean  = False;
                     Smooth_Weighten : Single   = 1;
                     AddMode         : TAddMode = AMSigned)            : TSingleMatrix;

}var
  RandxDSeed : Integer;
implementation


Function Rand1Db(x : Integer) : Byte;
Begin
  x := ((x + RandxDSeed) shl 26) xor x;
  Result := x * (x * x * 14549 + 2354743) mod 257;
  inc(RandxDSeed);
end;

Function Rand2Db(x, y : Integer) : Byte;
Begin
  x := x * 23 + y * 29 + RandxDSeed;
  x := (x shl 26) xor x;
  Result := x * (x * x * 14549 + 2354743) mod 257;
  inc(RandxDSeed);
end;

Function Rand1Df(x : Integer) : Single;
Begin
  x := ((x + RandxDSeed) shl 26) xor x;
  Result := (x * (x * x * 14549 + 2354743) and $7fffffff) * ( 1 / 2147483648);
  inc(RandxDSeed);
end;

Function Rand2Df(x, y : Integer) : Single;
Begin
  x := x * 23 + y * 29 + RandxDSeed;
  x := (x shl 26) xor x;
  Result := (x * (x * x * 14549 + 2354743) and $7fffffff) * ( 1 / 2147483648);
  inc(RandxDSeed);
end;

Function ClampToByte(Value : Integer) : Byte;
Begin
  If Value > 255 then
    Result := 255
  else If Value < 0 then
    Result := 0
  else
    Result := Value;

End;

Function ClampToNorm(Value : Single) : Single;
Begin
  If Value > 1 then
    Result := 1
  else If Value < 0 then
    Result := 0
  else
    Result := Value;

End;

Procedure Fexponentize(Arguments  : Array of Single;
                       var Matrix : TSingleMatrix;
                       Formula    : String);
var
  x, y    : Integer;
  Exponent: Single;
Begin
  Arguments[0] := 1 - Arguments[0];
  Arguments[1] := 1 - Arguments[1];
  If Arguments[1] = 0 then
    Arguments[1] := 0.001;
  For y := High(Matrix) downto 0 do
    For x := High(Matrix[0]) downto 0 do
    Begin
      Exponent := Matrix[y, x] - Arguments[0];
      If Exponent < 0 then Exponent := 0;
        Matrix[y, x] := 1 - exp(Exponent * ln(Arguments[1]));
    End;
End;

Procedure FMarble(Arguments  : Array of Single;
                  var Matrix : TSingleMatrix;
                  Formula    : String);
var
  x, y    : Integer;
Begin
  Arguments[1] := Arguments[1] * 2 * pi / Length(Matrix[0]);
  Arguments[2] := Arguments[2] * 2 * pi / Length(Matrix);
  For y := High(Matrix) downto 0 do
    For x := High(Matrix[0]) downto 0 do
      Matrix[y, x] := abs(cos(x * Arguments[1] + y * Arguments[2] + Matrix[y, x] * Arguments[0]));
End;

Procedure FWood(Arguments : Array of Single;
               var Matrix : TSingleMatrix;
               Formula    : String);
var
  x, y    : Integer;
  tmp     : Single;
Begin
  For y := High(Matrix) downto 0 do
    For x := High(Matrix[0]) downto 0 do
    Begin
      tmp          := Arguments[0] * round(Matrix[y, x] * 255);
      Matrix[y, x] := ((tmp / 255) - (round(tmp) div 255));
    End;
End;

Procedure FSmooth(Arguments  : Array of Single;
                  var Matrix : TSingleMatrix;
                  Formula    : String);
var
  x, y, z : Single;
Begin
  x := Arguments[0];
  y := Arguments[1];
  z := Arguments[2];
  SmoothMatrixM(Matrix, [0, 0, 0, 0, 0,
                         0, z, y, z, 0,
                         0, y, x, y, 0,
                         0, z, y, z, 0,
                         0, 0, 0, 0, 0]);
End;

Procedure FResize(Arguments  : Array of Single;
                  var Matrix : TSingleMatrix;
                  Formula    : String);
Begin
  Case Round(Arguments[2]) of
    0  : Matrix := BiCubResize (Matrix, Round(Arguments[0]), Round(Arguments[1]));
    1  : Matrix := BiCosResize (Matrix, Round(Arguments[0]), Round(Arguments[1]));
    2  : Matrix := BiQuadResize(Matrix, Round(Arguments[0]), Round(Arguments[1]));
    3  : Matrix := BiLinResize (Matrix, Round(Arguments[0]), Round(Arguments[1]));
    else Matrix := BiCubResize (Matrix, Round(Arguments[0]), Round(Arguments[1]));
  end;
End;

Procedure FInvert(Arguments  : Array of Single;
                  var Matrix : TSingleMatrix;
                  Formula    : String);
var
  x, y    : Integer;
Begin
  For y := High(Matrix) downto 0 do
    For x := High(Matrix[0]) downto 0 do
    Begin
      Matrix[y, x] := 1 - Matrix[y, x];
    End;
End;

Procedure FNone(Arguments  : Array of Single;
                var Matrix : TSingleMatrix;
                Formula    : String);
Begin
End;

Procedure FAdvanced(Arguments  : Array of Single;
                    var Matrix : TSingleMatrix;
                    Formula    : String);
Begin
  CompileAndExecute(Matrix, Formula);
End;
Function BiLinResize(Matrix : TSinglematrix;
                                 newWidth, newHeight : Cardinal) : TSinglematrix;
var
  x, y, i             : Integer;
  Xtmp, YTmp, YTmpOld,
  YTmpM2              : Integer;
  Factor              : Single;
  Data                : TSinglematrix;
  XTmps               : Array of Integer;
  XFactors            : Array of Single;
Begin
  Setlength(Result, newHeight, newWidth);
  SetLength(Matrix, Length(Matrix) + 1, Length(Matrix[0]) + 1);
  Setlength(Data, 2, newWidth);
  For y := High(Matrix) - 1 downto 0 do
    Matrix[y, high(Matrix[y])] := Matrix[y, 0];
  Matrix[High(Matrix)] := Matrix[0];

  For x := newwidth - 1 downto 0 do
  Begin
    Xtmp := trunc(x * High(Matrix)    / newWidth);
    Factor :=    (x * High(Matrix[0]) / newWidth - Xtmp);
    Data[0, x] := Matrix[0, Xtmp + 1] * Factor +
                  Matrix[0, Xtmp    ] * (1 - Factor);
  end;

  YTmpOld := - 1;
  setlength(XTmps   , newWidth);
  setlength(XFactors, newWidth);
  For x := newwidth - 1 downto 0 do
  Begin
    XFactors[x] :=      (x * High(Matrix[0]) / newWidth);
    Xtmps[x]    := trunc(XFactors[x]);
    XFactors[x] := XFactors[x] - XTmps[x];
  end;
  For y := 0 to newHeight - 1 do
  Begin
    YTmp := trunc(y * High(Matrix)/ NewHeight);
    YTmpM2 := (YTmp + 1) mod 2;
    If YTmp + 1 > YTmpOld then
    Begin
      YTmpOld := YTmp + 1;
      For x := newwidth - 1 downto 0 do
      Begin
         Data[YTmpM2, x] := Matrix[YTmp + 1, Xtmps[x] + 1] * XFactors[x] +
                            Matrix[YTmp + 1, Xtmps[x]    ] * (1 - XFactors[x]);
      End;
    end;

    Factor    :=   (y * High(Matrix) / newHeight - Ytmp);
    For x := newwidth - 1 downto 0 do
      Result[y, x] := Data[    YTmpM2, x] * Factor +
                      Data[1 - YTmpM2, x] * (1 - Factor);
  End;
End;
{Function BiLinResize(Matrix : TSingleMatrix;
                                 newWidth, newHeight : Cardinal) : TSingleMatrix;
var
  x, y, i             : Integer;
  Xtmp, YTmp, YTmpOld : Integer;
  Factor              : Single;
  Data                : TSingleMatrix;
Begin
  Setlength(Result, newHeight, newWidth);
  SetLength(Matrix, Length(Matrix) + 1, Length(Matrix[0]) + 1);
  Setlength(Data, 2, newWidth);
  For y := High(Matrix) - 1 downto 0 do
    Matrix[y, high(Matrix[y])] := Matrix[y, 0];
  Matrix[High(Matrix)] := Matrix[0];

  For x := newwidth - 1 downto 0 do
  Begin
    Xtmp := trunc(x * (Length(Matrix)    - 1) / newWidth);
    Factor :=    (x * (Length(Matrix[0]) - 1) / newWidth - Xtmp);
    Data[0, x] := Matrix[0, Xtmp + 1] * Factor +
                  Matrix[0, Xtmp    ] * (1 - Factor);
  end;

  YTmpOld := - 1;

  For y := 0 to newHeight - 1 do
  Begin
    YTmp := trunc(y * (Length(Matrix) - 1)/ NewHeight);
    If YTmp + 1 > YTmpOld then
    Begin
      YTmpOld := YTmp + 1;
      For x := newwidth - 1 downto 0 do
      Begin
        Xtmp := trunc(x * (Length(Matrix)    - 1) / newWidth);
        Factor :=    (x * (Length(Matrix[0]) - 1) / newWidth - Xtmp);
        Data[(YTmp + 1) mod 2, x] := Matrix[YTmp + 1, Xtmp + 1] * Factor +
                                     Matrix[YTmp + 1, Xtmp    ] * (1 - Factor);
      End;
    end;

    Factor :=   (y * (Length(Matrix) - 1) / newHeight - Ytmp);
    For x := newwidth - 1 downto 0 do
      Result[y, x] := Data[(YTmp + 1) mod 2, x] * Factor +
                      Data[ YTmp      mod 2, x] * (1 - Factor);
  End;
End;    }

Function BiQuadResize(Matrix : TSingleMatrix;
                      newWidth, newHeight : Cardinal) : TSingleMatrix;
var
  x, y, i             : Integer;
  Xtmp, YTmp, YTmpOld : Integer;
  Factor              : Single;
  Data                : TSingleMatrix;
Begin
  Setlength(Result, newHeight, newWidth);
  SetLength(Matrix, Length(Matrix) + 1, Length(Matrix[0]) + 1);
  Setlength(Data, 2, newWidth);
  For y := High(Matrix) - 1 downto 0 do
    Matrix[y, high(Matrix[y])] := Matrix[y, 0];
  Matrix[High(Matrix)] := Matrix[0];

  For x := newwidth - 1 downto 0 do
  Begin
    Xtmp := trunc(x * (Length(Matrix)    - 1) / newWidth);
    Factor :=    (x * (Length(Matrix[0]) - 1) / newWidth - Xtmp);
    If Factor > 0.5 then
    Begin
      Factor := 1 - Factor;
      Factor := 2 * Factor * Factor;
    End
    else
    Begin
      Factor := 2 * Factor * Factor;
      Factor := 1 - Factor;
    end;
    Data[0, x] := Matrix[0, Xtmp    ] * Factor +
                  Matrix[0, Xtmp + 1] * (1 - Factor);
  end;

  YTmpOld := - 1;

  For y := 0 to newHeight - 1 do
  Begin
    YTmp := trunc(y * (Length(Matrix) - 1)/ NewHeight);
    If YTmp + 1 > YTmpOld then
    Begin
      YTmpOld := YTmp + 1;
      For x := newwidth - 1 downto 0 do
      Begin
        Xtmp := trunc(x * (Length(Matrix)    - 1) / newWidth);
        Factor :=    (x * (Length(Matrix[0]) - 1) / newWidth - Xtmp);
        If Factor > 0.5 then
        Begin
          Factor := 1 - Factor;
          Factor := 2 * Factor * Factor;
        End
        else
        Begin
          Factor := 2 * Factor * Factor;
          Factor := 1 - Factor;
        end;
        Data[(YTmp + 1) mod 2, x] := Matrix[YTmp + 1, Xtmp    ] * Factor +
                                     Matrix[YTmp + 1, Xtmp + 1] * (1 - Factor);
      End;
    end;

    Factor :=   (y * (Length(Matrix) - 1) / newHeight - Ytmp);
    If Factor > 0.5 then
    Begin
      Factor := 1 - Factor;
      Factor := 2 * Factor * Factor;
    End
    else
    Begin
      Factor := 2 * Factor * Factor;
      Factor := 1 - Factor;
    end;


    For x := newwidth - 1 downto 0 do
      Result[y, x] := Data[ YTmp      mod 2, x] * Factor +
                      Data[(YTmp + 1) mod 2, x] * (1 - Factor);
  End;
End;

Function BiCosResize(Matrix : TSingleMatrix;
                     newWidth, newHeight : Cardinal) : TSingleMatrix;
var
  x, y, i            : Integer;
  Xtmp, YTmp, YTmpOld: Integer;
  Factor             : Single;
  Data               : TSingleMatrix;
Begin
  Setlength(Result, newHeight, newWidth);
  SetLength(Matrix, Length(Matrix) + 1, Length(Matrix[0]) + 1);
  Setlength(Data, 2, newWidth);
  For y := High(Matrix) - 1 downto 0 do
    Matrix[y, high(Matrix[y])] := Matrix[y, 0];
  Matrix[High(Matrix)] := Matrix[0];

  For x := newwidth - 1 downto 0 do
  Begin
    Xtmp := trunc(x * (Length(Matrix)    - 1) / newWidth);
    Factor :=    (x * (Length(Matrix[0]) - 1) / newWidth - Xtmp);
    Factor := pi * Factor;
    Factor := (1 - cos(Factor)) * 0.5;

    Data[0, x] := Matrix[0, Xtmp]     * (1 - Factor) +
                  Matrix[0, Xtmp + 1] * Factor;
  end;

  YTmpOld := - 1;

  For y := 0 to newHeight - 1 do
  Begin
    YTmp := trunc(y * (Length(Matrix) - 1)/ NewHeight);
    If YTmp + 1 > YTmpOld then
    Begin
      YTmpOld := YTmp + 1;
      For x := newwidth - 1 downto 0 do
      Begin
        Xtmp := trunc(x * (Length(Matrix)    - 1) / newWidth);
        Factor :=   (x * (Length(Matrix[0]) - 1) / newWidth - Xtmp);
        Factor := pi * Factor;
        Factor := (1 - cos(Factor)) * 0.5;
        Data[(YTmp + 1) mod 2, x] := Matrix[YTmp + 1, Xtmp    ] * (1 - Factor) +
                                     Matrix[YTmp + 1, Xtmp + 1] * Factor;
      End;
    end;

    Factor :=   (y * (Length(Matrix) - 1) / newHeight - Ytmp);
    Factor := pi * Factor;
    Factor := (1 - cos(Factor)) * 0.5;


    For x := newwidth - 1 downto 0 do
      Result[y, x] := Data[ YTmp      mod 2, x] * (1 - Factor) +
                      Data[(YTmp + 1) mod 2, x] * Factor;
  End;
End;


Function BiCubResize(Matrix : TSingleMatrix;
                     newWidth, newHeight : Cardinal) : TSingleMatrix;
var
  x, y, i            : Integer;
  Xtmp, YTmp, YTmpOld : Integer;
  Factor             : Single;
  P, Q, R, S         : Single;
  Data               : Array of Array of Single;
Begin
  Setlength(Result, newHeight, newWidth);
  SetLength(Matrix, Length(Matrix) + 3, Length(Matrix[0]) + 3);
  Setlength(Data, 4, newWidth);
  For y := High(Matrix) - 1 downto 0 do
  Begin
    Matrix[y, high(Matrix[y]) - 2] := Matrix[y, 0];
    Matrix[y, high(Matrix[y]) - 1] := Matrix[y, 1];
    Matrix[y, high(Matrix[y])    ] := Matrix[y, 2];
  End;
  Matrix[High(Matrix) - 2] := Matrix[0];
  Matrix[High(Matrix) - 1] := Matrix[1];
  Matrix[High(Matrix)    ] := Matrix[2];

  For y := 2 downto 0 do
    For x := newwidth - 1 downto 0 do
    Begin
      YTmp := (y + Length(Matrix) - 1) mod Length(Matrix);
      Xtmp := trunc(x * (Length(Matrix)    - 3) / newWidth) + 1;
      Factor :=    (x * (Length(Matrix[0]) - 3) / newWidth  + 1 - Xtmp);
      P      := (Matrix[YTmp, Xtmp + 2] - Matrix[YTmp, Xtmp + 1]) -
                (Matrix[YTmp, Xtmp - 1] - Matrix[YTmp, Xtmp    ]);
      Q      := (Matrix[YTmp, Xtmp - 1] - Matrix[YTmp, Xtmp    ]) - P;
      R      := (Matrix[YTmp, Xtmp + 1] - Matrix[YTmp, Xtmp - 1]);
      S      :=  Matrix[YTmp, Xtmp    ];
      Data[y, x] := P * Factor * Factor * Factor +
                    Q * Factor * Factor +
                    R * Factor +
                    S;
    end;

  YTmpOld := - 1;

  For y := 0 to newHeight - 1 do
  Begin
    YTmp := trunc(y * (Length(Matrix) - 3)/ NewHeight) + 1;
    If YTmp + 1 > YTmpOld then
    Begin
      YTmpOld := YTmp + 1;
      For x := newwidth - 1 downto 0 do
      Begin
        Xtmp := trunc(x * (Length(Matrix)    - 3) / newWidth) + 1;
        Factor :=    (x * (Length(Matrix[0]) - 3) / newWidth  + 1 - Xtmp);
        P      := (Matrix[YTmpOld, Xtmp + 2] - Matrix[YTmpOld, Xtmp + 1]) -
                  (Matrix[YTmpOld, Xtmp - 1] - Matrix[YTmpOld, Xtmp    ]);
        Q      := (Matrix[YTmpOld, Xtmp - 1] - Matrix[YTmpOld, Xtmp    ]) - P;
        R      := (Matrix[YTmpOld, Xtmp + 1] - Matrix[YTmpOld, Xtmp - 1]);
        S      :=  Matrix[YTmpOld, Xtmp    ];
        Data[(YTmpOld + 1) mod 4, x] := P * Factor * Factor * Factor +
                                        Q * Factor * Factor +
                                        R * Factor +
                                        S;
      End;
    end;

    Factor :=   (y * (Length(Matrix) - 3) / newHeight + 1 - Ytmp);


    For x := newwidth - 1 downto 0 do
    Begin
      P      := (Data[(YTmp + 2) mod 4, x] - Data[(YTmp + 1) mod 4, x]) -
                (Data[(YTmp + 3) mod 4, x] - Data[(YTmp    ) mod 4, x]);
      Q      := (Data[(YTmp + 3) mod 4, x] - Data[(YTmp    ) mod 4, x]) - P;
      R      := (Data[(YTmp + 1) mod 4, x] - Data[(YTmp + 3) mod 4, x]);
      S      :=  Data[(YTmp    ) mod 4, x];
      Result[y, x] := ClampToNorm(P * Factor * Factor * Factor +
                                  Q * Factor * Factor +
                                  R * Factor +
                                  S);

    End;
  End;
End;

Procedure smoothMatrix(var Matrix : TSingleMatrix; weighten : Single);
var
  x, y, i       : Integer;
  Xp, Xm, Yp, Ym,
  Xo, Yo        : Cardinal;
  Data          : TSingleMatrix;
Begin
  If (Length(Matrix) < 3) or (Length(Matrix[0]) < 3) then
    exit;
  setlength(Data, 3           , Length(Matrix[0]));
  move(Matrix[0, 0]           , Data[0, 0], Length(Matrix[0]) * 4);
  move(Matrix[1, 0]           , Data[1, 0], Length(Matrix[0]) * 4);
  move(Matrix[High(Matrix), 0], Data[2, 0], Length(Matrix[0]) * 4);
  For y := 0 to High(Matrix) do
  Begin
    For x := 0 to High(Matrix[0]) do
    Begin
      Xp :=  (x + 1)                      mod Length(Matrix[0]);
      Xm := ((x - 1) + Length(Matrix[0])) mod Length(Matrix[0]);
      Xo :=  (x    )                      mod Length(Matrix[0]);
      Yp := (Y + 1) mod 3;
      Ym := (Y + 2) mod 3;
      Yo := (Y    ) mod 3;
      Matrix[Y, X] := (Data[Ym, Xm]  +               {upper-left  corner}
                       Data[Ym, Xp]  +               {upper-right corner}
                       Data[Yp, Xm]  +               {lower-left  corner}
                       Data[Yp, Xp]) *0.66 +         {lower-right corner}
                      (Data[Ym, Xo]  +               {upper neighbor}
                       Data[Yo, Xm]  +               {left  neighbor}
                       Data[Yo, Xp]  +               {right neighbor}
                       Data[Yp, Xo]) +               {lower neighbor}
                      (Data[Yo, Xo]  * weighten) /  {pixel itself}
                      (weighten + 6.66);
    End;
    move(Matrix[(Y + 2) mod Length(Matrix), 0], Data[(Y + 2) mod 3, 0], Length(Matrix[0]) * 4);
  end;
end;

Procedure smoothMatrixM(var Matrix : TSingleMatrix; weighten : Array of Single);
var
  x, y, i       : Integer;
  Xp, Xpp, Xm, Xmm,
  Yp, Ypp, Ym, Ymm,
  Xo, Yo        : Cardinal;
  Data          : TSingleMatrix;
  weighsum      : Single;
Begin
  If{ (Length(Matrix)   < 5) or (Length(Matrix[0])   < 5) or }
     (Length(weighten) < 25) then
    exit;
  setlength(Data, 5                       , Length(Matrix[0]));
  move(Matrix[0, 0]                       , Data[0, 0], Length(Matrix[0]) * 4);
  move(Matrix[1 mod Length(Matrix), 0]    , Data[1, 0], Length(Matrix[0]) * 4);
  move(Matrix[2 mod Length(Matrix), 0]    , Data[2, 0], Length(Matrix[0]) * 4);
  move(Matrix[max(High(Matrix) - 1, 0), 0], Data[3, 0], Length(Matrix[0]) * 4);
  move(Matrix[High(Matrix), 0]            , Data[4, 0], Length(Matrix[0]) * 4);
  WeighSum := 0;
  For y := 0 to 4 do
    For x := 0 to 4 do
      weighSum := weighSum + weighten[y * 5 + x];
  If weighsum = 0 then
    weighsum := 0.001;
  For y := 0 to High(Matrix) do
  Begin
    For x := 0 to High(Matrix[0]) do
    Begin
      Xp :=  (x + 2)                      mod Length(Matrix[0]);
      Xpp:=  (x + 2)                      mod Length(Matrix[0]);
      Xm := ((x - 2) + Length(Matrix[0])) mod Length(Matrix[0]);
      Xmm:= ((x - 2) + Length(Matrix[0])) mod Length(Matrix[0]);
      Xo :=  (x    )                      mod Length(Matrix[0]);
      Yp := (Y + 1) mod 5;
      Ypp:= (Y + 2) mod 5;
      Ym := (Y + 4) mod 5;
      Ymm:= (Y + 3) mod 5;
      Yo := (Y    ) mod 5;
      Matrix[Y, X] := (Data[Ymm, Xmm] * weighten[    0] +
                       Data[Ymm, Xm]  * weighten[    1] +
                       Data[Ymm, Xo]  * weighten[    2] +
                       Data[Ymm, Xp]  * weighten[    3] +
                       Data[Ymm, Xpp] * weighten[    4] +

                       Data[Ym, Xmm] * weighten[5    ] +
                       Data[Ym, Xpp] * weighten[5 + 4] +
                       Data[Yo, Xmm] * weighten[10   ] +
                       Data[Yo, Xpp] * weighten[10+ 4] +
                       Data[Yp, Xmm] * weighten[15   ] +
                       Data[Yp, Xpp] * weighten[15+ 4] +

                       Data[Ypp, Xmm] * weighten[20+ 0] +
                       Data[Ypp, Xm]  * weighten[20+ 1] +
                       Data[Ypp, Xo]  * weighten[20+ 2] +
                       Data[Ypp, Xp]  * weighten[20+ 3] +
                       Data[Ypp, Xpp] * weighten[20+ 4] +


                       Data[Ym, Xm] * weighten[5 + 1] + {upper-left  corner}
                       Data[Ym, Xp] * weighten[5 + 3] + {upper-right corner}
                       Data[Yp, Xm] * weighten[15+ 1] + {lower-left  corner}
                       Data[Yp, Xp] * weighten[15+ 3] + {lower-right corner}

                       Data[Ym, Xo] * weighten[5 + 2] + {upper neighbor}
                       Data[Yo, Xm] * weighten[10+ 1] + {left  neighbor}
                       Data[Yo, Xp] * weighten[10+ 3] + {right neighbor}
                       Data[Yp, Xo] * weighten[15+ 2] + {lower neighbor}

                       Data[Yo, Xo] * weighten[10+ 2]) /  {pixel itself}
                       weighsum;
     End;
    move(Matrix[(Y + 3) mod Length(Matrix), 0], Data[(Y + 3) mod 5, 0], Length(Matrix[0]) * 4);
  end;
end;

Procedure NoiseMatrix(var Matrix : TSingleMatrix;Seed : Integer);
var
  x, y : Integer;
Begin
  RandxDSeed := Seed;
  For y := High(Matrix) downto 0 do
  Begin
    For x := High(Matrix[0]) downto 0 do
      Matrix[Y, X] := Rand2Df(x, y);
  end;
End;

Function ValInArray(       Value : Integer; Arr : Array of Integer;
                    var Position : Integer) : Boolean;
var
  i : integer;
Begin
  Result := False;
  Position := -1;
  For i := 0 to High(Arr) do
    If Arr[i] = Value then
    Begin
      Result   := True;
      Position := i;
      exit;
    end;
end;

Function PerlinNoise(Persistance      : Single   ; StartOctave, Octaves : Byte;
                     Seed             : Integer  ;
               {      FunctionArguments: Array of Single;  }
                     PerlinFunc       : Array of TPerlFunc;
                     Layers           : Array of TSingleMatrix;
                     LayerOctaves     : Array of Integer;
                     Smooth_Weighten  : Array of Single;
                     Smooth           : Boolean       = False;

                     Translate        : Single        = 0;
                     Scale            : Single        = 1;
                     ResizeMethod     : TResizeMethod = RMCosinus;
                     AddMode          : TAddMode      = AMSigned)    : TSingleMatrix;
var
  Matrices   : Array of TSingleMatrix;
  i, x, y    : Integer;
  LayerPos   : Integer;
  Persists   : Array of Single;
  tmp        : single;
  Factor     : Single;
Begin
  If octaves = 0 then
    exit;
  setlength(Matrices , 1);//Octaves);
  setlength(Persists , Octaves);


  Setlength(Result, 1 shl (Octaves + StartOctave), 1 shl (Octaves + StartOctave));
  For y := 0 to High(Result) do
    For x := 0 to High(Result) do
      Result[y, x] := 0;

  If AddMode = AMNormalized then
  Begin
    Factor := 1;
    For i := 1 to Octaves - 1 do
      Factor := Factor + Persists[i];
  end;

  For i := 0 to Octaves - 1 do
  Begin
    Persists[i] := exp(i * ln(Persistance));
    Setlength(Matrices[0], 1 shl (i + StartOctave), 1 shl (i + StartOctave));
    If not ValInArray(i + StartOctave, LayerOctaves, LayerPos) then
    Begin
      NoiseMatrix(Matrices[0], Seed);

      If Smooth then
    //      SmoothMatrix(Matrices[i], Smooth_weighten);
          SmoothMatrixM(Matrices[0],Smooth_weighten);
    end
    else
      Matrices[0] := Layers[LayerPos];

    If (Length(Matrices[0])    <> 1 shl (Octaves + StartOctave)) or
       (Length(Matrices[0, 0]) <> 1 shl (Octaves + StartOctave)) then
      Case ResizeMethod of
        RMLinear    : Matrices[0] := BiLinResize(Matrices[0],
                                           1 shl (Octaves + StartOctave),
                                           1 shl (Octaves + StartOctave));
        RMQuadratic : Matrices[0] := BiQuadResize(Matrices[0],
                                           1 shl (Octaves + StartOctave),
                                           1 shl (Octaves + StartOctave));
        RMCosinus   : Matrices[0] := BiCosResize(Matrices[0],
                                           1 shl (Octaves + StartOctave),
                                           1 shl (Octaves + StartOctave));
        RMCubic     : Matrices[0] := BiCubResize(Matrices[0],
                                           1 shl (Octaves + StartOctave),
                                           1 shl (Octaves + StartOctave));
      end;

    case Addmode of
      AMSigned :
      Begin
        If i > 0 then
        Begin
          For y := High(Result) downto 0 do
            For x := 0 to High(Result[0]) do
              Result[y, x] := Result[y, x] + (Matrices[0, Y, X] - 0.5) * Persists[i];
        end
        else
        Begin
          For y := High(Result) downto 0 do
            For x := 0 to High(Result[0]) do
              Result[y, x] := Result[y, x] + Matrices[0, Y, X];
        End;
      end;
      AMUnSigned :
      Begin
        For y := High(Result) downto 0 do
          For x := 0 to High(Result[0]) do
            Result[y, x] := Result[y, x] + Matrices[0, Y, X] * Persists[i];
      end;
      AMAbs :
      Begin
        For y := High(Result) downto 0 do
          For x := 0 to High(Result[0]) do
            Result[y, x] := Result[y, x] + abs((Matrices[0, Y, X] - 0.5) * Persists[i]);
      end;
      AMNormalized :
      Begin

        For y := High(Result) downto 0 do
          For x := 0 to High(Result[0]) do
            Result[y, x] := Result[y, x] + (Matrices[0, Y, X]) * Persists[i];
      end;
    end;
   // Setlength(Matrices[i], 0, 0);
    Seed := (Seed shl 11) xor Seed;
  End;


  If AddMode <> AMNormalized then
  Begin
    For y := High(Result) downto 0 do
      For x := 0 to High(Result[0]) do
        Result[y, x] := translate + Scale * Result[y, x];
  end
  else
  Begin
    For y := High(Result) downto 0 do
      For x := 0 to High(Result[0]) do
        Result[y, x] := translate + Scale * Result[y, x] / Factor;
  end;



  {  Setlength(Result, Length(Matrices[0, 0]), Length(Matrices[0, 0]));
  case Addmode of
    AMSigned :
    Begin
      For y := High(Result) downto 0 do
      Begin
        For x := 0 to High(Result[0]) do
        Begin
          tmp := Matrices[0, Y, X];
          For i := 1 to Octaves - 1 do
            tmp := tmp + (Matrices[i, Y, X] - 0.5) * Persists[i];
          Result[y, x] := ClampToNorm(translate + Scale * tmp);
        end;
      End;
    end;
    AMUnSigned :
    Begin
      For y := High(Result) downto 0 do
      Begin
        For x := 0 to High(Result[0]) do
        Begin
          tmp := Matrices[0, Y, X];
          For i := 1 to Octaves - 1 do
            tmp := tmp + Matrices[i, Y, X] * Persists[i];
          Result[y, x] := ClampToNorm(translate + Scale * tmp);
        end;
      End;
    end;
    AMAbs :
    Begin
      For y := High(Result) downto 0 do
      Begin
        For x := 0 to High(Result[0]) do
        Begin
          tmp := abs(Matrices[0, Y, X] - 0.5);
          For i := 1 to Octaves - 1 do
            tmp := tmp + abs((Matrices[i, Y, X] - 0.5) * Persists[i]);
          Result[y, x] := ClampToNorm(translate + Scale * tmp);
        end;
      End;
    end;
    AMNormalized :
    Begin
      Factor := 1;
      For i := 1 to Octaves - 1 do
        Factor := Factor + Persists[i];

      For y := High(Result) downto 0 do
      Begin
        For x := 0 to High(Result[0]) do
        Begin
          tmp := Matrices[0, Y, X];
          For i := 1 to Octaves - 1 do
            tmp := tmp + (Matrices[i, Y, X]) * Persists[i];
          Result[y, x] := ClampToNorm(translate + Scale * tmp / Factor);
        end;
      End;
    end;
  end;     }
  Setlength(Matrices, 0, 0, 0);
  For i := 0 to high(PerlinFunc) do
    PerlinFunc[i].Func(PerlinFunc[i].Arguments, Result, PerlinFunc[i].Formular);

End;

Function PerlinNoiseBmpE(Persistance      : Single   ; StartOctave, Octaves : Byte;
                     Seed             : Integer  ;
                     Palette          : Array of Cardinal;
                     PerlinFunc       : Array of TPerlFunc;
                     Bmps             : Array of TBitmap;
                     BmpOctaves       : Array of Integer;
                     Smooth_Weighten  : Array of Single;
                     Smooth           : Boolean       = False;

                     Translate        : Single        = 0;
                     Scale            : Single        = 1;
                     ResizeMethod     : TResizeMethod = RMCosinus;
                     AddMode          : TAddMode      = AMSigned)  : TBitmap;
var
  Layers : Array of TSingleMatrix;
  i      : Integer;
Begin
  Setlength(Layers, Length(Bmps));
  For i := 0 to High(Bmps) do
  Begin
    Layers[i] := BmpToMatrix(Bmps[i], 1, pf32Bit, Palette);
  end;

  Result := MatrixToBmp(PerlinNoise(Persistance    , StartOctave      , Octaves,
                                    Seed           , PerlinFunc       ,
                                    Layers         , BmpOctaves       ,
                                    Smooth_Weighten, Smooth           ,
                                    translate      , Scale            ,
                                    ResizeMethod, AddMode),
                        1, pf24Bit, Palette);

end;


Function MatrixToBmp(Matrix : TSingleMatrix; Offset : Byte; PF : TPixelformat;
                     Palette: Array of Cardinal   ) : TBitmap;
var
  x, y, i : Integer;
  Ba      : PByteArray;
  PFOff   : Byte;
Begin
  Result := TBitmap.create;
  Result.Height := Length(Matrix);
  Result.Width  := Length(Matrix[0]) div OffSet;
  Result.Pixelformat := PF;
  If (PF = pf8Bit)  and (Offset = 1) or
     (PF = pf16Bit) and (Offset = 2) or
     (PF = pf24Bit) and (Offset = 3) or
     (PF = pf32Bit) and (Offset = 4) then
    For y := Result.Height - 1 downto 0 do
    Begin
      Ba := Result.ScanLine[Y];
      Move(Matrix[Y], Ba, Result.width * Offset);
    End
  else
    if not(Offset = 1) then
      exit
  else
  Begin
    Result.PixelFormat := pf32Bit;
    If Length(Palette) < exp(Offset * ln(256)) then
    Begin
      For y := Result.Height - 1 downto 0 do
      Begin
        Ba := Result.ScanLine[Y];
        For x := Result.Width - 1 downto 0 do
        Begin
          For i := 0 to 3 do
            Ba[x * 4 + i] := Round(ClampToNorm(Matrix[Y, X]) * 255);
        end;
      End;
    end
    else
    Begin
      For y := Result.Height - 1 downto 0 do
      Begin
        Ba := Result.ScanLine[Y];
        For x := Result.Width - 1 downto 0 do
        Begin

          Move(Palette[Round(ClampToNorm(Matrix[Y, X]) * 255)],  Ba[x * 4], 4);
        {  For i := 0 to 3 do
            Ba[x * 4 + i] := Matrix[Y, X];  }
        end;
      End;
    End;

    Result.PixelFormat := PF;

  End;
End;

Function BmpToMatrix(Bmp : TBitmap; Offset : Byte; PF : TPixelformat;
                     Palette: Array of Cardinal ) : TSingleMatrix;
var
  x, y, i : Integer;
  Ba      : PByteArray;
  PFOff   : Byte;
Begin
  Setlength(Result, Bmp.Height, Bmp.Width * Offset);
  Bmp.Pixelformat := PF;

  If (PF = pf8Bit)  and (Offset = 1) or
     (PF = pf16Bit) and (Offset = 2) or
     (PF = pf24Bit) and (Offset = 3) or
     (PF = pf32Bit) and (Offset = 4) then
    For y := Bmp.Height - 1 downto 0 do
    Begin
      Ba := Bmp.ScanLine[Y];
      Move(Ba, Result[Y], Bmp.width * Offset);
    End
  else
    if not(Offset = 1) then
      exit
  else
  Begin
    Bmp.PixelFormat := pf32Bit;
//    If Length(Palette) < exp(Offset * ln(256)) then
  //  Begin
      For y := Bmp.Height - 1 downto 0 do
      Begin
        Ba := Bmp.ScanLine[Y];
        For x := Bmp.Width - 1 downto 0 do
        Begin
        //  For i := 0 to 3 do
          Result[Y, X] := Ba[x * 4] / 255;
        end;
      End;
   // end
   (* else
    Begin
      For y := Bmp.Height - 1 downto 0 do
      Begin
        Ba := Bmp.ScanLine[Y];
        For x := Bmp.Width - 1 downto 0 do
        Begin

          Move( Ba[x * 4], Result[Y, X], 4);
        {  For i := 0 to 3 do
            Ba[x * 4 + i] := Matrix[Y, X];  }
        end;
      End;
    End; *)

    Bmp.PixelFormat := PF;

  End;
End;

initialization
  RandxDSeed := 0;
end.






{Function PerlinNoiseBmp(Persistance     : Single  ; StartOctave, Octaves : Byte;
                        Seed            : Integer ; ResizeMethod         : TResizeMethod = RMCosinus;
                        Smooth          : Boolean = False;
                        Smooth_Weighten : Single  = 1)                   : TBitmap;

var
  Matrices : Array of TSingleMatrix;
  Matrices2: Array of TSingleMatrix;
  i, x, y  : Integer;
  BAR      : PByteArray;
  Persists : Array of Single;
  tmp      : single;
Begin
  If octaves = 0 then
    exit;
  setlength(Matrices , Octaves);
  setlength(Persists , Octaves);
  For i := 0 to Octaves - 1 do
  Begin
    Persists[i] := exp(i * ln(Persistance));
    Setlength(Matrices[i], 1 shl (i + StartOctave), 1 shl (i + StartOctave));
    NoiseMatrix(Matrices[i], Seed);
    If Smooth then
      SmoothMatrix(Matrices[i], Smooth_weighten);
    Case ResizeMethod of
      RMLinear    : Matrices[i] := BiLinResize(Matrices[i],
                                         1 shl (Octaves + StartOctave),
                                         1 shl (Octaves + StartOctave));
      RMQuadratic : Matrices[i] := BiQuadResize(Matrices[i],
                                         1 shl (Octaves + StartOctave),
                                         1 shl (Octaves + StartOctave));
      RMCosinus   : Matrices[i] := BiCosResize(Matrices[i],
                                         1 shl (Octaves + StartOctave),
                                         1 shl (Octaves + StartOctave));
      RMCubic     : Matrices[i] := BiCubResize(Matrices[i],
                                         1 shl (Octaves + StartOctave),
                                         1 shl (Octaves + StartOctave));
    end;
    Seed := (Seed shl 11) xor Seed;
  End;
  Result := TBitmap.create;
  Result.PixelFormat := pf24bit;
  Result.Width  := Length(Matrices[0, 0]);
  Result.Height := Result.Width;
  For y := Result.Height - 1 downto 0 do
  Begin
    BAR := Result.ScanLine[Y];
    For x := 0 to Result.width - 1 do
    Begin
      tmp := Matrices[0, Y, X];
      For i := 1 to Octaves - 1 do
        tmp := tmp + (Matrices[i, Y, X] - 128) * Persists[i];
      Bar[x * 3] := ClampToByte(Round(tmp));
      For i := 1 to 2 do
        Bar[x * 3 + i] := Bar[x * 3];
    end;
  End;
End;       }












(* For Later use


Procedure smoothMatrixO(var Matrix : TByteMatrix; Offset : Byte; weighten : Single);
var
  x, y, i       : Integer;
  Xp, Xm, Yp, Ym,
  Xo, Yo        : Cardinal;
  Ba            : PByteArray;
  Data          : Array of Array of Byte;
Begin
  If (Length(Matrix) < 3) or (Length(Matrix[0]) / Offset < 3) then
    exit;
  setlength(Data, 3           , Length(Matrix[0]));
  move(Matrix[0, 0]           , Data[0, 0], Length(Matrix[0]));
  move(Matrix[1, 0]           , Data[1, 0], Length(Matrix[0]));
  move(Matrix[High(Matrix), 0], Data[2, 0], Length(Matrix[0]));
  For y := 0 to High(Matrix) do
  Begin
    For x := 0 to round(High(Matrix[0]) / offset) do
    Begin
      Xp := ((x + 1) * Offset)                     mod Length(Matrix[0]);
      Xm := ((x - 1) * Offset + Length(Matrix[0])) mod Length(Matrix[0]);
      Xo := ((x    ) * Offset)                     mod Length(Matrix[0]);
      Yp := (Y + 1) mod 3;
      Ym := (Y + 2) mod 3;
      Yo := (Y    ) mod 3;
      For i := 0 to Offset - 1 do
        Matrix[Y, X * offset + i] :=
                        Round(((Data[Ym, Xm + i]  +               {upper-left  corner}
                                Data[Ym, Xp + i]  +               {upper-right corner}
                                Data[Yp, Xm + i]  +               {lower-left  corner}
                                Data[Yp, Xp + i]) *0.66 +         {lower-right corner}
                               (Data[Ym, Xo + i]  +               {upper neighbor}
                                Data[Yo, Xm + i]  +               {left  neighbor}
                                Data[Yo, Xp + i]  +               {right neighbor}
                                Data[Yp, Xo + i]) +               {lower neighbor}
                               (Data[Yo, Xo + i]  * weighten)) /  {pixel itself}
                               (weighten + 6.66));
    End;
    move(Matrix[(Y + 2) mod Length(Matrix), 0], Data[(Y + 2) mod 3, 0], Length(Matrix[0]));
  end;
end;

Procedure NoiseMatrixO(var Matrix : TByteMatrix; Offset : Byte; Seed : Integer);
var
  x, y, i : Integer;
Begin
  RandxDSeed := Seed;
  For y := High(Matrix) downto 0 do
  Begin
    For x := High(Matrix[0]) downto 0 do
    Begin
      Matrix[Y, X * Offset] := Rand2Db(x , y);
      For i := 0 to Offset - 1 do
        Matrix[Y, X * Offset + i] := Rand2Db(x * offset + i, y);
    End;
  end;
End;


Function BiQuadResizeO(Matrix : TByteMatrix; Offset : Byte;
                                   newWidth, newHeight : Cardinal) : TByteMatrix;
var
  x, y, i            : Integer;
  tmp, YTmp, YTmpOld : Integer;
  Factor             : Single;
  Data               : Array of Array of Byte;
Begin
  Setlength(Result, newHeight, newWidth * Offset);
  SetLength(Matrix, Length(Matrix) + 1, Length(Matrix[0]) + 1);
  Setlength(Data, 2, newWidth * Offset);
  For y := High(Matrix) - 1 downto 0 do
    Matrix[y, high(Matrix[y])] := Matrix[y, 0];
  Matrix[High(Matrix)] := Matrix[0];

  For x := newwidth - 1 downto 0 do
  Begin
    tmp := trunc(x * (Length(Matrix)    - 1) / newWidth);
    Factor :=   (x * (Length(Matrix[0]) - 1) / newWidth - tmp) / offset;
    If Factor > 0.5 then
    Begin
      Factor := 1 - Factor;
      Factor := 2 * Factor * Factor;
    End
    else
    Begin
      Factor := 2 * Factor * Factor;
      Factor := 1 - Factor;
    end;
    For i := Offset - 1 downto 0 do
    Begin
      Data[0, x * Offset + i] := Round(Matrix[0, tmp          + i] * Factor +
                                       Matrix[0, tmp + Offset + i] * (1 - Factor));
    End;
  end;

  YTmpOld := - 1;

  For y := 0 to newHeight - 1 do
  Begin
    YTmp := trunc(y * (Length(Matrix) - 1)/ NewHeight);
    If YTmp + 1 > YTmpOld then
    Begin
      YTmpOld := YTmp + 1;
      For x := newwidth - 1 downto 0 do
      Begin
        tmp := trunc(x * (Length(Matrix)    - 1) / newWidth);
        Factor :=   (x * (Length(Matrix[0]) - 1) / newWidth - tmp) / offset;
        If Factor > 0.5 then
        Begin
          Factor := 1 - Factor;
          Factor := 2 * Factor * Factor;
        End
        else
        Begin
          Factor := 2 * Factor * Factor;
          Factor := 1 - Factor;
        end;
        For i := Offset - 1 downto 0 do
        Begin
          Data[(YTmp + 1) mod 2, x * Offset + i] := Round(Matrix[YTmp + 1, tmp          + i] * Factor +
                                                          Matrix[YTmp + 1, tmp + Offset + i] * (1 - Factor));
       End;
      End;
    end;

    Factor :=   (y * (Length(Matrix) - 1) / newHeight - Ytmp);
    If Factor > 0.5 then
    Begin
      Factor := 1 - Factor;
      Factor := 2 * Factor * Factor;
    End
    else
    Begin
      Factor := 2 * Factor * Factor;
      Factor := 1 - Factor;
    end;


    For x := newwidth - 1 downto 0 do
      For i := Offset - 1 downto 0 do
      Begin
        Result[y, x * Offset + i] := Round(Data[ YTmp      mod 2, x * Offset + i] * Factor +
                                           Data[(YTmp + 1) mod 2, x * Offset + i] * (1 - Factor));
      End;
  End;
End;

Function BiCosResizeO(Matrix : TByteMatrix; Offset : Byte;
                                  newWidth, newHeight : Cardinal) : TByteMatrix;
var
  x, y, i            : Integer;
  tmp, YTmp, YTmpOld : Integer;
  Factor             : Single;
  Data               : Array of Array of Byte;
Begin
  Setlength(Result, newHeight, newWidth * Offset);
  SetLength(Matrix, Length(Matrix) + 1, Length(Matrix[0]) + 1);
  Setlength(Data, 2, newWidth * Offset);
  For y := High(Matrix) - 1 downto 0 do
    Matrix[y, high(Matrix[y])] := Matrix[y, 0];
  Matrix[High(Matrix)] := Matrix[0];

  For x := newwidth - 1 downto 0 do
  Begin
    tmp := trunc(x * (Length(Matrix)    - 1) / newWidth);
    Factor :=   (x * (Length(Matrix[0]) - 1) / newWidth - tmp) / offset;
    Factor := pi * Factor;
    Factor := (1 - cos(Factor)) * 0.5;

    For i := Offset - 1 downto 0 do
    Begin
      Data[0, x * Offset + i] := Round(Matrix[0, tmp          + i] * (1 - Factor) +
                                       Matrix[0, tmp + Offset + i] * Factor);
    End;
  end;

  YTmpOld := - 1;

  For y := 0 to newHeight - 1 do
  Begin
    YTmp := trunc(y * (Length(Matrix) - 1)/ NewHeight);
    If YTmp + 1 > YTmpOld then
    Begin
      YTmpOld := YTmp + 1;
      For x := newwidth - 1 downto 0 do
      Begin
        tmp := trunc(x * (Length(Matrix)    - 1) / newWidth);
        Factor :=   (x * (Length(Matrix[0]) - 1) / newWidth - tmp) / offset;
        Factor := pi * Factor;
        Factor := (1 - cos(Factor)) * 0.5;
        For i := Offset - 1 downto 0 do
        Begin
          Data[(YTmp + 1) mod 2, x * Offset + i] := Round(Matrix[YTmp + 1, tmp          + i] * (1 - Factor) +
                                                          Matrix[YTmp + 1, tmp + Offset + i] * Factor);
        End;
      End;
    end;

    Factor :=   (y * (Length(Matrix) - 1) / newHeight - Ytmp);
    Factor := pi * Factor;
    Factor := (1 - cos(Factor)) * 0.5;


    For x := newwidth - 1 downto 0 do
      For i := Offset - 1 downto 0 do
      Begin
        Result[y, x * Offset + i] := Round(Data[ YTmp      mod 2,  x * Offset + i] * (1 - Factor) +
                                           Data[(YTmp + 1) mod 2, x * Offset + i]  * Factor);
      End;
  End;
End;

Function PerlinNoiseO(Persistance     : Single ; StartOctave, Octaves, Offset : Byte;
                      Seed            : Integer; Smooth                       : Boolean = False;
                      Smooth_Weighten : Single = 1)                           : TBitmap;
var
  Matrices : Array of TByteMatrix;
  Matrices2: Array of TByteMatrix;
  i, x, y  : Integer;
  BAR      : PByteArray;
  Persists : Array of Single;
  tmp      : single;
Begin
  If octaves = 0 then
    exit;
  setlength(Matrices , Octaves);
  setlength(Persists , Octaves);
  For i := 0 to Octaves - 1 do
  Begin
    Persists[i] := exp(i * ln(Persistance));
    Setlength(Matrices[i], 1 shl (i + StartOctave), 1 shl (i + StartOctave));
    NoiseMatrixO(Matrices[i], Offset, Seed);
    If Smooth then
      SmoothMatrixO(Matrices[i], Offset, Smooth_weighten);
    Matrices[i] := BiCosResizeO(Matrices[i], Offset, 1 shl (Octaves + StartOctave),
                                                                 1 shl (Octaves + StartOctave));
    Seed := (Seed shl 11) xor Seed;
  End;
  Result := TBitmap.create;
  Result.PixelFormat := pf24bit;
  Result.Width  := Length(Matrices[0, 0]);
  Result.Height := Result.Width;
  For y := Result.Height - 1 downto 0 do
  Begin
    BAR := Result.ScanLine[Y];
    For x := 0 to Result.width - 1 do
    Begin
      tmp := Matrices[0, Y, X];
      For i := 1 to Octaves - 1 do
        tmp := tmp + (Matrices[i, Y, X] - 128) * Persists[i];
      Bar[x * 3] := max(min(Round(tmp), 255), 0);
      For i := 1 to 2 do
        Bar[x * 3 + i] := Bar[x * 3];
    end;
  End;
End;

Function PixResizeO(Matrix : TByteMatrix; Offset : Byte; newWidth, NewHeight : Integer) : TByteMatrix;
Var
  x, y, i : Integer;
Begin
  Setlength(Result, newHeight, newWidth * Offset);
  For   y := newHeight - 1 downto 0 do
    For x := newWidth  - 1 downto 0 do
      For i := 0 to Offset - 1 do
        Result[y, x * Offset + i] := Matrix[trunc(y / newHeight * high(Matrix)),
                                            trunc(x / newWidth  * high(Matrix[0])) + i];
End;
*)